@@ -123,14 +123,6 @@ object KyuubiSQLConf {
       .bytesConf(ByteUnit.BYTE)
       .createOptional
 
-  val WATCHDOG_FORCED_MAXOUTPUTROWS =
-    buildConf("spark.sql.watchdog.forcedMaxOutputRows")
-      .doc("Add ForcedMaxOutputRows rule to avoid huge output rows of non-limit query " +
-        "unexpectedly, it's optional that works with defined")
-      .version("1.4.0")
-      .intConf
-      .createOptional
-
   val DROP_IGNORE_NONEXISTENT =
     buildConf("spark.sql.optimizer.dropIgnoreNonExistent")
       .doc("Do not report an error if DROP DATABASE/TABLE/VIEW/FUNCTION/PARTITION specifies " +
@@ -240,6 +232,22 @@ object KyuubiSQLConf {
       .stringConf
       .createOptional
 
+  val DYNAMIC_SHUFFLE_PARTITIONS =
+    buildConf("spark.sql.optimizer.dynamicShufflePartitions")
+      .doc("If true, adjust the number of shuffle partitions dynamically based on the job" +
+        " input size. The new number of partitions is the maximum input size" +
+        " divided by `spark.sql.adaptive.advisoryPartitionSizeInBytes`.")
+      .version("1.9.0")
+      .booleanConf
+      .createWithDefault(false)
+
+  val DYNAMIC_SHUFFLE_PARTITIONS_MAX_NUM =
+    buildConf("spark.sql.optimizer.dynamicShufflePartitions.maxNum")
+      .doc("The maximum partition number of DynamicShufflePartitions.")
+      .version("1.9.0")
+      .intConf
+      .createWithDefault(2000)
+
   val SCRIPT_TRANSFORMATION_ENABLED =
     buildConf("spark.sql.execution.scriptTransformation.enabled")
       .doc("When false, script transformation is not allowed.")
