@@ -45,6 +45,10 @@ class RangerSparkExtension extends (SparkSessionExtensions => Unit) {
 
   override def apply(v1: SparkSessionExtensions): Unit = {
     v1.injectCheckRule(AuthzConfigurationChecker)
+    // RuleFunctionAuthorization should use injectCheckRule instead of injectOptimizerRule,
+    // because ConstantFolding will optimize deterministic UDFs with foldable
+    // inputs (e.g., literals), replacing them with their results and bypassing permission checks.
+    v1.injectCheckRule(RuleFunctionAuthorization)
     v1.injectResolutionRule(_ => RuleReplaceShowObjectCommands)
     v1.injectResolutionRule(_ => RuleApplyPermanentViewMarker)
     v1.injectResolutionRule(_ => RuleApplyTypeOfMarker)
