@@ -43,6 +43,16 @@ object TableCommands extends CommandSpecs[TableCommandSpec] {
     TableCommandSpec(cmd, Seq(tableDesc), ALTERTABLE_PROPERTIES, uriDescs = uriDescs)
   }
 
+  val RenameTable = {
+    val cmd = "org.apache.spark.sql.catalyst.plans.logical.RenameTable"
+    val oldTableDesc =
+      TableDesc(
+        "child",
+        classOf[ResolvedTableTableExtractor])
+
+    TableCommandSpec(cmd, Seq(oldTableDesc), ALTERTABLE_RENAME)
+  }
+
   val AlterTableAddColumns = {
     val cmd = "org.apache.spark.sql.execution.command.AlterTableAddColumnsCommand"
     val columnDesc = ColumnDesc("colsToAdd", classOf[StructFieldSeqColumnExtractor])
@@ -122,13 +132,12 @@ object TableCommands extends CommandSpecs[TableCommandSpec] {
     TableCommandSpec(cmd, Seq(oldTableD), ALTERTABLE_RENAME)
   }
 
-  // this is for spark 3.1 or below
-  val AlterTableRecoverPartitions = {
-    val cmd = "org.apache.spark.sql.execution.command.AlterTableRecoverPartitionsCommand"
+  val RepairTable = {
+    val cmd = "org.apache.spark.sql.execution.command.RepairTableCommand"
     TableCommandSpec(cmd, Seq(tableNameDesc), MSCK)
   }
 
-  val RepairTable = {
+  val RepairTableV2 = {
     val cmd = "org.apache.spark.sql.catalyst.plans.logical.RepairTable"
     TableCommandSpec(cmd, Seq(resolvedTableDesc), MSCK)
   }
@@ -167,6 +176,11 @@ object TableCommands extends CommandSpecs[TableCommandSpec] {
     Seq(tableNameDesc),
     ALTERTABLE_PROPERTIES)
 
+  val UnsetTableProperties = TableCommandSpec(
+    "org.apache.spark.sql.catalyst.plans.logical.UnsetTableProperties",
+    Seq(resolvedTableDesc),
+    ALTERTABLE_PROPERTIES)
+
   val AlterTableUnsetProperties = AlterTableSetProperties.copy(classname =
     "org.apache.spark.sql.execution.command.AlterTableUnsetPropertiesCommand")
 
@@ -652,9 +666,8 @@ object TableCommands extends CommandSpecs[TableCommandSpec] {
     TableCommandSpec(cmd, Nil, ADD, uriDescs = Seq(uriDesc))
   }
 
-  // For spark-3.1
-  val AddFileCommand = {
-    val cmd = "org.apache.spark.sql.execution.command.AddFileCommand"
+  val AddJarCommand = {
+    val cmd = "org.apache.spark.sql.execution.command.AddJarCommand"
     val uriDesc = UriDesc("path", classOf[StringURIExtractor], isInput = true)
     TableCommandSpec(cmd, Nil, ADD, uriDescs = Seq(uriDesc))
   }
@@ -663,8 +676,7 @@ object TableCommands extends CommandSpecs[TableCommandSpec] {
     AddArchivesCommand,
     AddArchivesCommand.copy(classname = "org.apache.spark.sql.execution.command.AddFilesCommand"),
     AddArchivesCommand.copy(classname = "org.apache.spark.sql.execution.command.AddJarsCommand"),
-    AddFileCommand,
-    AddFileCommand.copy(classname = "org.apache.spark.sql.execution.command.AddJarCommand"),
+    AddJarCommand,
     AddPartitions,
     DropPartitions,
     RenamePartitions,
@@ -674,20 +686,19 @@ object TableCommands extends CommandSpecs[TableCommandSpec] {
     DropColumns,
     ReplaceColumns,
     RenameColumn,
+    RenameTable,
     AlterTable,
     AlterTableAddColumns,
     AlterTableAddPartition,
     AlterTableChangeColumn,
     AlterTableDropPartition,
     AlterTableRename,
-    AlterTableRecoverPartitions,
-    AlterTableRecoverPartitions.copy(classname =
-      "org.apache.spark.sql.execution.command.RepairTableCommand"),
     AlterTableRenamePartition,
     AlterTableSerDeProperties,
     AlterTableSetLocation,
     AlterTableSetProperties,
     AlterTableUnsetProperties,
+    UnsetTableProperties,
     AlterViewAs,
     AnalyzeColumn,
     AnalyzePartition,
@@ -732,6 +743,7 @@ object TableCommands extends CommandSpecs[TableCommandSpec] {
     OverwriteByExpression,
     OverwritePartitionsDynamic,
     RepairTable,
+    RepairTableV2,
     RefreshTable,
     RefreshTableV2,
     RefreshTable3d0,
