@@ -23,6 +23,7 @@ import java.nio.file.Path
 import scala.util.Try
 
 import org.apache.hadoop.security.UserGroupInformation
+import org.apache.spark.SparkConf
 import org.apache.spark.sql.{DataFrame, Row, SparkSessionExtensions}
 import org.apache.spark.sql.catalyst.analysis.NoSuchTableException
 import org.apache.spark.sql.catalyst.catalog.HiveTableRelation
@@ -37,21 +38,52 @@ import org.scalatest.BeforeAndAfterAll
 import org.scalatest.funsuite.AnyFunSuite
 
 import org.apache.kyuubi.Utils
+import org.apache.kyuubi.plugin.lineage.Lineage
+import org.apache.kyuubi.plugin.lineage.helper.SparkSQLLineageParseHelper
 import org.apache.kyuubi.plugin.spark.authz.{AccessControlException, SparkSessionProvider}
+import org.apache.kyuubi.plugin.spark.authz.MysqlContainerEnv
 import org.apache.kyuubi.plugin.spark.authz.RangerTestNamespace._
 import org.apache.kyuubi.plugin.spark.authz.RangerTestUsers._
 import org.apache.kyuubi.plugin.spark.authz.rule.Authorization.KYUUBI_AUTHZ_TAG
 import org.apache.kyuubi.plugin.spark.authz.util.AuthZUtils._
 import org.apache.kyuubi.util.AssertionUtils._
 import org.apache.kyuubi.util.reflect.ReflectUtils._
+
 abstract class RangerSparkExtensionSuite extends AnyFunSuite
-  with SparkSessionProvider with BeforeAndAfterAll {
+  with SparkSessionProvider with BeforeAndAfterAll with MysqlContainerEnv {
   // scalastyle:on
   override protected val extension: SparkSessionExtensions => Unit = new RangerSparkExtension
 
+  var mysqlJdbcUrl = ""
+  var mysqlUsername = ""
+  var mysqlPassword = ""
+  var driverClassName = ""
+
+  override def getMysqlJdbcUrl: String = mysqlJdbcUrl
+
+  override def getMysqlUsername: String = mysqlUsername
+
+  override def getMysqlPassword: String = mysqlPassword
+
+  override def getDriverClassName: String = driverClassName
+
   override def afterAll(): Unit = {
     spark.stop()
     super.afterAll()
+    if (useMysqlEnv) {
+      stopEngine()
+    }
+  }
+
+  override def beforeAll(): Unit = {
+    if (useMysqlEnv) {
+      startEngine()
+      this.mysqlJdbcUrl = containerDef.jdbcUrl
+      this.mysqlUsername = containerDef.username
+      this.mysqlPassword = containerDef.password
+      this.driverClassName = containerDef.driverClassName
+    }
+    super.beforeAll()
   }
 
   protected def errorMessage(
@@ -242,13 +274,19 @@ abstract class RangerSparkExtensionSuite extends AnyFunSuite
   test("auth: functions") {
     val db = defaultDb
     val func = "func"
-    val create0 = s"CREATE FUNCTION IF NOT EXISTS $db.$func AS 'abc.mnl.xyz'"
-    doAs(
-      kent, {
+    withCleanTmpResources(Seq(
+      (func, "function"))) {
+      val create0 = s"CREATE FUNCTION IF NOT EXISTS $db.$func AS 'abc.mnl.xyz'"
+      doAs(bob) {
         val e = intercept[AccessControlException](sql(create0))
-        assert(e.getMessage === errorMessage("create", "default/func"))
-      })
-    doAs(admin, assert(Try(sql(create0)).isSuccess))
+        assert(e.getMessage === errorMessage("create", s"$db/$func"))
+      }
+      doAs(kent) {
+        val e = intercept[AccessControlException](sql(create0))
+        assert(e.getMessage === errorMessage("create", s"$db/$func"))
+      }
+      doAs(admin, assert(Try(sql(create0)).isSuccess))
+    }
   }
 
   test("show tables") {
@@ -491,6 +529,9 @@ class InMemoryCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite
 
 class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   override protected val catalogImpl: String = "hive"
+  override protected val extraSparkConf: SparkConf = new SparkConf()
+    .set("spark.kyuubi.authz.udf.enabled", "true")
+
   test("table stats must be specified") {
     val table = "hive_src"
     withCleanTmpResources(Seq((table, "table"))) {
@@ -650,37 +691,34 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   }
 
   test("[KYUUBI #3411] skip checking cache table") {
-    if (isSparkV32OrGreater) { // cache table sql supported since 3.2.0
+    val db1 = defaultDb
+    val srcTable1 = "hive_src1"
+    val cacheTable1 = "cacheTable1"
+    val cacheTable2 = "cacheTable2"
+    val cacheTable3 = "cacheTable3"
+    val cacheTable4 = "cacheTable4"
 
-      val db1 = defaultDb
-      val srcTable1 = "hive_src1"
-      val cacheTable1 = "cacheTable1"
-      val cacheTable2 = "cacheTable2"
-      val cacheTable3 = "cacheTable3"
-      val cacheTable4 = "cacheTable4"
+    withCleanTmpResources(Seq(
+      (s"$db1.$srcTable1", "table"),
+      (s"$db1.$cacheTable1", "cache"),
+      (s"$db1.$cacheTable2", "cache"),
+      (s"$db1.$cacheTable3", "cache"),
+      (s"$db1.$cacheTable4", "cache"))) {
 
-      withCleanTmpResources(Seq(
-        (s"$db1.$srcTable1", "table"),
-        (s"$db1.$cacheTable1", "cache"),
-        (s"$db1.$cacheTable2", "cache"),
-        (s"$db1.$cacheTable3", "cache"),
-        (s"$db1.$cacheTable4", "cache"))) {
+      doAs(
+        admin,
+        sql(s"CREATE TABLE IF NOT EXISTS $db1.$srcTable1" +
+          s" (id int, name string, city string)"))
 
-        doAs(
-          admin,
-          sql(s"CREATE TABLE IF NOT EXISTS $db1.$srcTable1" +
-            s" (id int, name string, city string)"))
-
-        withSingleCallEnabled {
-          val e1 = intercept[AccessControlException](
-            doAs(someone, sql(s"CACHE TABLE $cacheTable2 select * from $db1.$srcTable1")))
-          assert(
-            e1.getMessage.contains(s"does not have [select] privilege on " +
-              s"[$db1/$srcTable1/city,$db1/$srcTable1/id,$db1/$srcTable1/name]"))
-        }
-        doAs(admin, sql(s"CACHE TABLE $cacheTable3 SELECT 1 AS a, 2 AS b "))
-        doAs(someone, sql(s"CACHE TABLE $cacheTable4 select 1 as a, 2 as b "))
+      withSingleCallEnabled {
+        val e1 = intercept[AccessControlException](
+          doAs(someone, sql(s"CACHE TABLE $cacheTable2 select * from $db1.$srcTable1")))
+        assert(
+          e1.getMessage.contains(s"does not have [select] privilege on " +
+            s"[$db1/$srcTable1/city,$db1/$srcTable1/id,$db1/$srcTable1/name]"))
       }
+      doAs(admin, sql(s"CACHE TABLE $cacheTable3 SELECT 1 AS a, 2 AS b "))
+      doAs(someone, sql(s"CACHE TABLE $cacheTable4 select 1 as a, 2 as b "))
     }
   }
 
@@ -965,7 +1003,6 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   }
 
   test("[KYUUBI #5503][AUTHZ] Check plan auth checked should not set tag to all child nodes") {
-    assume(isSparkV32OrGreater, "Spark 3.1 not support lateral subquery.")
     val db1 = defaultDb
     val table1 = "table1"
     val table2 = "table2"
@@ -1157,11 +1194,7 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
   test("Add resource command") {
     withTempDir { path =>
       withSingleCallEnabled {
-        val supportedCommand = if (isSparkV32OrGreater) {
-          Seq("JAR", "FILE", "ARCHIVE")
-        } else {
-          Seq("JAR", "FILE")
-        }
+        val supportedCommand = Seq("JAR", "FILE", "ARCHIVE")
         supportedCommand.foreach { cmd =>
           interceptEndsWith[AccessControlException](
             doAs(someone, sql(s"ADD $cmd $path")))(
@@ -1485,4 +1518,62 @@ class HiveCatalogRangerSparkExtensionSuite extends RangerSparkExtensionSuite {
       }
     }
   }
+
+  test("Test view lineage") {
+    def extractLineage(sql: String): Lineage = {
+      val parsed = spark.sessionState.sqlParser.parsePlan(sql)
+      val qe = spark.sessionState.executePlan(parsed)
+      val analyzed = qe.analyzed
+      SparkSQLLineageParseHelper(spark).transformToLineage(0, analyzed).get
+    }
+
+    val db1 = defaultDb
+    val table1 = "table1"
+    val view1 = "view1"
+    withSingleCallEnabled {
+      withCleanTmpResources(Seq((s"$db1.$table1", "table"), (s"$db1.$view1", "view"))) {
+        doAs(admin, sql(s"CREATE TABLE IF NOT EXISTS $db1.$table1 (id int, scope int)"))
+        doAs(admin, sql(s"CREATE VIEW $db1.$view1 AS SELECT * FROM $db1.$table1"))
+
+        val lineage = doAs(
+          admin,
+          extractLineage(s"SELECT id FROM $db1.$view1 WHERE id > 1"))
+        assert(lineage.inputTables.size == 1)
+        assert(lineage.inputTables.head === s"spark_catalog.$db1.$table1")
+        assert(lineage.columnLineage.size == 1)
+        assert(lineage.columnLineage.head.originalColumns.head === s"spark_catalog.$db1.$table1.id")
+      }
+    }
+  }
+
+  test("[KYUUBI #7186] Introduce RuleFunctionAuthorization") {
+    val db = defaultDb
+    val kyuubiFunc = "kyuubi_func1"
+    withCleanTmpResources(Seq(
+      (kyuubiFunc, "function"))) {
+      val createKyuubiFunc =
+        s"""
+           |CREATE FUNCTION IF NOT EXISTS
+           |  $db.$kyuubiFunc
+           |  AS 'org.apache.hadoop.hive.ql.udf.generic.GenericUDFMaskHash'
+           |""".stripMargin
+      doAs(kent) {
+        val e = intercept[AccessControlException](sql(createKyuubiFunc))
+        assert(e.getMessage === errorMessage("create", s"$db/$kyuubiFunc"))
+      }
+      doAs(bob, assert(Try(sql(createKyuubiFunc)).isSuccess))
+      doAs(admin, assert(Try(sql(createKyuubiFunc)).isSuccess))
+
+      val selectKyuubiFunc =
+        s"""
+           |SELECT $db.$kyuubiFunc("KYUUBUI_TEST_STRING")""".stripMargin
+      doAs(alice) {
+        val e = intercept[AccessControlException](sql(selectKyuubiFunc))
+        assert(e.getMessage === errorMessage("select", s"$db/$kyuubiFunc"))
+      }
+      doAs(kent, assert(Try(sql(selectKyuubiFunc)).isSuccess))
+      doAs(bob, assert(Try(sql(selectKyuubiFunc)).isSuccess))
+      doAs(admin, assert(Try(sql(selectKyuubiFunc)).isSuccess))
+    }
+  }
 }
