@@ -28,13 +28,10 @@ import org.apache.kyuubi.KyuubiFunSuite
 import org.apache.kyuubi.events.EventBus
 import org.apache.kyuubi.plugin.lineage.Lineage
 import org.apache.kyuubi.plugin.lineage.dispatcher.{OperationLineageKyuubiEvent, OperationLineageSparkEvent}
-import org.apache.kyuubi.plugin.lineage.helper.SparkListenerHelper.SPARK_RUNTIME_VERSION
 
 class OperationLineageEventSuite extends KyuubiFunSuite with SparkListenerExtensionTest {
 
-  val catalogName =
-    if (SPARK_RUNTIME_VERSION <= "3.1") "org.apache.spark.sql.connector.InMemoryTableCatalog"
-    else "org.apache.spark.sql.connector.catalog.InMemoryTableCatalog"
+  val catalogName = "org.apache.spark.sql.connector.catalog.InMemoryTableCatalog"
 
   override protected val catalogImpl: String = "hive"
 
