@@ -91,12 +91,8 @@ class SparkOperationSuite extends WithSparkSQLEngine with HiveMetadataTests with
       .add("c15", "struct<X: bigint,Y: double>", nullable = true, "15")
       .add("c16", "binary", nullable = false, "16")
       .add("c17", "struct<X: string>", nullable = true, "17")
-
-    // since spark3.3.0
-    if (SPARK_ENGINE_RUNTIME_VERSION >= "3.3") {
-      schema = schema.add("c18", "interval day", nullable = true, "18")
-        .add("c19", "interval year", nullable = true, "19")
-    }
+      .add("c18", "interval day", nullable = true, "18")
+      .add("c19", "interval year", nullable = true, "19")
     // since spark3.4.0
     if (SPARK_ENGINE_RUNTIME_VERSION >= "3.4") {
       schema = schema.add("c20", "timestamp_ntz", nullable = true, "20")
