@@ -17,10 +17,10 @@
 
 package org.apache.kyuubi.engine.trino.operation
 
-import org.apache.kyuubi.config.KyuubiConf.{ENGINE_TRINO_CONNECTION_CATALOG, OPERATION_INCREMENTAL_COLLECT}
+import org.apache.kyuubi.config.KyuubiConf.{ENGINE_TRINO_CONNECTION_CATALOG, ENGINE_TRINO_OPERATION_INCREMENTAL_COLLECT}
 
 class TrinoOperationIncrementCollectSuite extends TrinoOperationSuite {
   override def withKyuubiConf: Map[String, String] = Map(
     ENGINE_TRINO_CONNECTION_CATALOG.key -> "memory",
-    OPERATION_INCREMENTAL_COLLECT.key -> "true")
+    ENGINE_TRINO_OPERATION_INCREMENTAL_COLLECT.key -> "true")
 }
