@@ -17,15 +17,15 @@
 
 package org.apache.kyuubi.it.hive.operation
 
-import org.apache.hive.service.rpc.thrift.{TGetInfoReq, TGetInfoType}
-
 import org.apache.kyuubi.{HiveEngineTests, Utils, WithKyuubiServer}
 import org.apache.kyuubi.config.KyuubiConf
 import org.apache.kyuubi.config.KyuubiConf._
+import org.apache.kyuubi.shaded.hive.service.rpc.thrift.{TGetInfoReq, TGetInfoType}
+import org.apache.kyuubi.util.JavaUtils
 
 class KyuubiOperationHiveEnginePerUserSuite extends WithKyuubiServer with HiveEngineTests {
 
-  val kyuubiHome: String = Utils.getCodeSourceLocation(getClass).split("integration-tests").head
+  val kyuubiHome: String = JavaUtils.getCodeSourceLocation(getClass).split("integration-tests").head
 
   override protected val conf: KyuubiConf = {
     val metastore = Utils.createTempDir(prefix = getClass.getSimpleName)
@@ -62,6 +62,20 @@ class KyuubiOperationHiveEnginePerUserSuite extends WithKyuubiServer with HiveEn
     }
   }
 
+  test("[KYUUBI #5865] Hive engine CLI_ODBC_KEYWORDS") {
+    withSessionConf(Map(KyuubiConf.SERVER_INFO_PROVIDER.key -> "ENGINE"))()() {
+      withSessionHandle { (client, handle) =>
+        val req = new TGetInfoReq()
+        req.setSessionHandle(handle)
+        req.setInfoType(TGetInfoType.CLI_ODBC_KEYWORDS)
+        val value = client.GetInfo(req).getInfoValue.getStringValue
+        assert(value.contains("DATABASE") || value === "Unimplemented")
+        // excluded keywords
+        assert(!value.contains("ADD"))
+      }
+    }
+  }
+
   test("kyuubi defined function - system_user, session_user") {
     withJdbcStatement("hive_engine_test") { statement =>
       val rs = statement.executeQuery("SELECT system_user(), session_user()")
