@@ -35,6 +35,11 @@ object ShareLevel extends Enumeration {
    * In this level, An APP will be shared for all sessions created by a user's default group
    */
   GROUP,
+  /**
+   * In this level, All sessions of the same engine type from one Kyuubi server's will share
+   * one single APP launched on the same server
+   */
+  SERVER_LOCAL,
   /**
    * In this level, All sessions from one or more Kyuubi server's will share one single APP
    */
