@@ -18,7 +18,7 @@
 package org.apache.kyuubi.operation
 
 import java.io.IOException
-import java.util.concurrent.{Future, ScheduledExecutorService, TimeUnit}
+import java.util.concurrent.{Future, ScheduledFuture}
 import java.util.concurrent.locks.ReentrantLock
 
 import scala.collection.JavaConverters._
@@ -32,7 +32,6 @@ import org.apache.kyuubi.operation.OperationState._
 import org.apache.kyuubi.operation.log.OperationLog
 import org.apache.kyuubi.session.Session
 import org.apache.kyuubi.shaded.hive.service.rpc.thrift.{TFetchResultsResp, TGetResultSetMetadataResp, TProgressUpdateResp, TProtocolVersion, TStatus, TStatusCode}
-import org.apache.kyuubi.util.ThreadUtils
 
 abstract class AbstractOperation(session: Session) extends Operation with Logging {
 
@@ -45,7 +44,7 @@ abstract class AbstractOperation(session: Session) extends Operation with Loggin
 
   final private[kyuubi] val statementId = handle.identifier.toString
 
-  private var statementTimeoutCleaner: Option[ScheduledExecutorService] = None
+  private var timeoutFuture: Option[ScheduledFuture[_]] = None
 
   private val lock: ReentrantLock = new ReentrantLock()
 
@@ -60,16 +59,29 @@ abstract class AbstractOperation(session: Session) extends Operation with Loggin
 
   protected def addTimeoutMonitor(queryTimeout: Long): Unit = {
     if (queryTimeout > 0) {
-      val timeoutExecutor =
-        ThreadUtils.newDaemonSingleThreadScheduledExecutor("query-timeout-thread", false)
-      val action: Runnable = () => cleanup(OperationState.TIMEOUT)
-      timeoutExecutor.schedule(action, queryTimeout, TimeUnit.SECONDS)
-      statementTimeoutCleaner = Some(timeoutExecutor)
+      val action: Runnable = () =>
+        // Clients less than version 2.1 have no HIVE-4924 Patch,
+        // no queryTimeout parameter and no TIMEOUT status.
+        // When the server enables kyuubi.operation.query.timeout,
+        // this will cause the client of the lower version to get stuck.
+        // Check thrift protocol version <= HIVE_CLI_SERVICE_PROTOCOL_V8(Hive 2.1.0),
+        // convert TIMEDOUT_STATE to CANCELED.
+        if (isHive21OrLower) {
+          cleanup(OperationState.CANCELED)
+        } else {
+          cleanup(OperationState.TIMEOUT)
+        }
+
+      val future = session.sessionManager.operationManager.scheduleTimeout(action, queryTimeout)
+      timeoutFuture = Some(future)
     }
   }
 
   protected def shutdownTimeoutMonitor(): Unit = {
-    statementTimeoutCleaner.foreach(_.shutdown())
+    timeoutFuture.foreach { future =>
+      session.sessionManager.operationManager.cancelTimeout(future)
+    }
+    timeoutFuture = None
   }
 
   override def getOperationLog: Option[OperationLog] = None
@@ -276,4 +288,8 @@ abstract class AbstractOperation(session: Session) extends Operation with Loggin
       }
     }
   }
+
+  protected def isHive21OrLower: Boolean = {
+    getProtocolVersion.getValue <= TProtocolVersion.HIVE_CLI_SERVICE_PROTOCOL_V8.getValue
+  }
 }
