@@ -18,7 +18,6 @@
 package org.apache.kyuubi
 
 import java.io.{File, IOException}
-import java.net.InetAddress
 import java.nio.file.{Files, Paths}
 import java.security.PrivilegedExceptionAction
 import java.util.Properties
@@ -118,15 +117,6 @@ class UtilsSuite extends KyuubiFunSuite {
       })
   }
 
-  test("findLocalInetAddress") {
-    val address = InetAddress.getLocalHost
-    if (!address.isLoopbackAddress) {
-      assert(Utils.findLocalInetAddress === InetAddress.getLocalHost)
-    } else {
-      assert(Utils.findLocalInetAddress !== InetAddress.getLocalHost)
-    }
-  }
-
   test("getAbsolutePathFromWork") {
     val workDir = System.getenv("KYUUBI_WORK_DIR_ROOT")
     val path1 = "path1"
