@@ -17,12 +17,14 @@
 
 package org.apache.kyuubi.service.authentication
 
+import java.security.Principal
 import javax.security.sasl.AuthenticationException
 
 import org.apache.kyuubi.Logging
+import org.apache.kyuubi.service.authentication.UserDefineTokenAuthenticationProviderImpl.VALID_TOKEN
 
 class UserDefineAuthenticationProviderImpl()
-  extends PasswdAuthenticationProvider with Logging {
+  extends PasswdAuthenticationProvider with TokenAuthenticationProvider with Logging {
 
   override def authenticate(user: String, password: String): Unit = {
     if (user == "user" && password == "password") {
@@ -31,4 +33,18 @@ class UserDefineAuthenticationProviderImpl()
       throw new AuthenticationException("Username or password is not valid!")
     }
   }
+
+  override def authenticate(credential: TokenCredential): Principal = {
+    val clientIp = credential.extraInfo.get(Credential.CLIENT_IP_KEY)
+    if (credential.token == VALID_TOKEN) {
+      info(s"Success log in of token: ${credential.token} with clientIp: $clientIp")
+      new BasicPrincipal("test")
+    } else {
+      throw new AuthenticationException("Token is not valid!")
+    }
+  }
+}
+
+object UserDefineTokenAuthenticationProviderImpl {
+  val VALID_TOKEN = "token"
 }
