@@ -89,13 +89,7 @@ import org.apache.commons.cli.ParseException;
 import org.apache.commons.io.IOUtils;
 import org.apache.hadoop.fs.FSDataInputStream;
 import org.apache.hive.beeline.common.util.ShutdownHookManager;
-import org.apache.hive.beeline.hs2connection.BeelineConfFileParseException;
-import org.apache.hive.beeline.hs2connection.BeelineSiteParseException;
-import org.apache.hive.beeline.hs2connection.BeelineSiteParser;
-import org.apache.hive.beeline.hs2connection.HS2ConnectionFileParser;
-import org.apache.hive.beeline.hs2connection.HS2ConnectionFileUtils;
-import org.apache.hive.beeline.hs2connection.HiveSiteHS2ConnectionFileParser;
-import org.apache.hive.beeline.hs2connection.UserHS2ConnectionFileParser;
+import org.apache.hive.beeline.hs2connection.*;
 import org.apache.kyuubi.jdbc.hive.JdbcConnectionParams;
 import org.apache.kyuubi.jdbc.hive.JdbcUriParseException;
 import org.apache.kyuubi.jdbc.hive.Utils;
@@ -165,6 +159,7 @@ public class BeeLine implements Closeable {
 
   private static final String HIVE_VAR_PREFIX = "--hivevar";
   private static final String HIVE_CONF_PREFIX = "--hiveconf";
+  private static final String CONF_PREFIX = "--conf";
   private static final String PROP_FILE_PREFIX = "--property-file";
   static final String PASSWD_MASK = "[passwd stripped]";
 
@@ -180,6 +175,8 @@ public class BeeLine implements Closeable {
             "tsv", new DeprecatedSeparatedValuesOutputFormat(this, '\t'),
             "xmlattr", new XMLAttributeOutputFormat(this),
             "xmlelements", new XMLElementOutputFormat(this),
+            "json", new JSONOutputFormat(this),
+            "jsonfile", new JSONFileOutputFormat(this),
           });
 
   private List<String> supportedLocalDriver =
@@ -381,6 +378,15 @@ public class BeeLine implements Closeable {
             .withDescription("Use value for given property")
             .create());
 
+    // conf option --conf
+    options.addOption(
+        OptionBuilder.withValueSeparator()
+            .hasArgs(2)
+            .withArgName("property=value")
+            .withLongOpt("conf")
+            .withDescription("Alias of --hiveconf")
+            .create());
+
     // --property-file <file>
     options.addOption(
         OptionBuilder.hasArg()
@@ -675,7 +681,8 @@ public class BeeLine implements Closeable {
     private boolean isBeeLineOpt(String arg) {
       return arg.startsWith("--")
           && !(HIVE_VAR_PREFIX.equals(arg)
-              || (HIVE_CONF_PREFIX.equals(arg))
+              || HIVE_CONF_PREFIX.equals(arg)
+              || CONF_PREFIX.equals(arg)
               || "--help".equals(arg)
               || PROP_FILE_PREFIX.equals(arg));
     }
@@ -755,6 +762,11 @@ public class BeeLine implements Closeable {
       setHiveConfVar(key, hiveConfs.getProperty(key));
     }
 
+    Properties confs = cl.getOptionProperties("conf");
+    for (String key : confs.stringPropertyNames()) {
+      setHiveConfVar(key, confs.getProperty(key));
+    }
+
     driver = cl.getOptionValue("d");
     auth = cl.getOptionValue("a");
     user = cl.getOptionValue("n");
@@ -952,7 +964,6 @@ public class BeeLine implements Closeable {
   /*
    * Attempts to make a connection using default HS2 connection config file if available
    * if there connection is not made return false
-   *
    */
   private boolean defaultBeelineConnect(CommandLine cl) {
     String url;
@@ -970,11 +981,9 @@ public class BeeLine implements Closeable {
   }
 
   private String getDefaultConnectionUrl(CommandLine cl) throws BeelineConfFileParseException {
-    Properties mergedConnectionProperties = new Properties();
     JdbcConnectionParams jdbcConnectionParams = null;
     BeelineSiteParser beelineSiteParser = getUserBeelineSiteParser();
     UserHS2ConnectionFileParser userHS2ConnFileParser = getUserHS2ConnFileParser();
-    Properties userConnectionProperties = new Properties();
 
     if (!userHS2ConnFileParser.configExists() && !beelineSiteParser.configExists()) {
       // nothing to do if there is no user HS2 connection configuration file
@@ -1005,11 +1014,13 @@ public class BeeLine implements Closeable {
       }
     }
 
+    Properties userConnectionProperties = new Properties();
     if (userHS2ConnFileParser.configExists()) {
       // get the connection properties from user specific config file
       userConnectionProperties = userHS2ConnFileParser.getConnectionProperties();
     }
 
+    Properties mergedConnectionProperties = userConnectionProperties;
     if (jdbcConnectionParams != null) {
       String userName = cl.getOptionValue("n");
       if (userName != null) {
@@ -1022,25 +1033,30 @@ public class BeeLine implements Closeable {
       mergedConnectionProperties =
           HS2ConnectionFileUtils.mergeUserConnectionPropertiesAndBeelineSite(
               userConnectionProperties, jdbcConnectionParams);
-    } else {
-      mergedConnectionProperties = userConnectionProperties;
     }
 
-    // load the HS2 connection url properties from hive-site.xml if it is present in the classpath
+    Properties serverConnectionProperties = new Properties();
+    KyuubiConfFileParser kyuubiConfFileParser = getKyuubiConfFileParser();
     HS2ConnectionFileParser hiveSiteParser = getHiveSiteHS2ConnectionFileParser();
-    Properties hiveSiteConnectionProperties = hiveSiteParser.getConnectionProperties();
-    // add/override properties found from hive-site with user-specific properties
+    if (kyuubiConfFileParser.configExists()) {
+      // load the Kyuubi connection url properties from kyuubi-defaults.conf if present
+      serverConnectionProperties = kyuubiConfFileParser.getConnectionProperties();
+    } else if (hiveSiteParser.configExists()) {
+      // load the HS2 connection url properties from hive-site.xml if present
+      serverConnectionProperties = hiveSiteParser.getConnectionProperties();
+    }
+    // add/override properties found from kyuubi-defaults.conf with user-specific properties
     for (String key : mergedConnectionProperties.stringPropertyNames()) {
-      if (hiveSiteConnectionProperties.containsKey(key)) {
+      if (serverConnectionProperties.containsKey(key)) {
         debug(
             "Overriding connection url property "
                 + key
                 + " from user connection configuration file");
       }
-      hiveSiteConnectionProperties.setProperty(key, mergedConnectionProperties.getProperty(key));
+      serverConnectionProperties.setProperty(key, mergedConnectionProperties.getProperty(key));
     }
     // return the url based on the aggregated connection properties
-    return HS2ConnectionFileUtils.getUrl(hiveSiteConnectionProperties);
+    return HS2ConnectionFileUtils.getUrl(serverConnectionProperties);
   }
 
   /*
@@ -1067,6 +1083,11 @@ public class BeeLine implements Closeable {
     return new HiveSiteHS2ConnectionFileParser();
   }
 
+  @VisibleForTesting
+  public KyuubiConfFileParser getKyuubiConfFileParser() {
+    return new KyuubiConfFileParser();
+  }
+
   int runInit() {
     String[] initFiles = getOpts().getInitFiles();
 
