@@ -27,8 +27,6 @@ import java.lang.reflect.InvocationHandler;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.lang.reflect.Proxy;
-import java.net.InetAddress;
-import java.net.UnknownHostException;
 import java.nio.ByteBuffer;
 import java.nio.charset.StandardCharsets;
 import java.security.*;
@@ -148,7 +146,11 @@ public class KyuubiConnection implements SQLConnection, KyuubiLoggable {
     // hive_conf_list -> hiveConfMap
     // hive_var_list -> hiveVarMap
     if (isKerberosAuthMode()) {
-      host = Utils.getCanonicalHostName(connParams.getHost());
+      if (isEnableCanonicalHostnameCheck()) {
+        host = Utils.getCanonicalHostName(connParams.getHost());
+      } else {
+        host = connParams.getHost();
+      }
     } else {
       host = connParams.getHost();
     }
@@ -215,7 +217,7 @@ public class KyuubiConnection implements SQLConnection, KyuubiLoggable {
           }
           // Update with new values
           jdbcUriString = connParams.getJdbcUriString();
-          if (isKerberosAuthMode()) {
+          if (isKerberosAuthMode() && isEnableCanonicalHostnameCheck()) {
             host = Utils.getCanonicalHostName(connParams.getHost());
           } else {
             host = connParams.getHost();
@@ -835,11 +837,7 @@ public class KyuubiConnection implements SQLConnection, KyuubiLoggable {
           "set:hiveconf:hive.server2.thrift.resultset.default.fetch.size",
           Integer.toString(fetchSize));
     }
-    try {
-      openConf.put("kyuubi.client.ipAddress", InetAddress.getLocalHost().getHostAddress());
-    } catch (UnknownHostException e) {
-      LOG.debug("Error getting Kyuubi session local client ip address", e);
-    }
+    openConf.put("kyuubi.client.ipAddress", Utils.CLIENT_IP_ADDRESS);
     openConf.put(Utils.KYUUBI_CLIENT_VERSION_KEY, Utils.getVersion());
     openReq.setConfiguration(openConf);
 
@@ -1009,6 +1007,12 @@ public class KyuubiConnection implements SQLConnection, KyuubiLoggable {
     return isSaslAuthMode() && hasSessionValue(AUTH_PRINCIPAL);
   }
 
+  private boolean isEnableCanonicalHostnameCheck() {
+    return Boolean.parseBoolean(
+        sessConfMap.getOrDefault(
+            JdbcConnectionParams.AUTH_KERBEROS_ENABLE_CANONICAL_HOSTNAME_CHECK, "true"));
+  }
+
   private Subject createSubject() {
     if (isKeytabAuthMode()) {
       String principal = sessConfMap.get(AUTH_KYUUBI_CLIENT_PRINCIPAL);
