@@ -58,6 +58,10 @@ class MetricsSystem extends CompositeService("MetricsSystem") {
     meter.mark(value)
   }
 
+  def getGauge(name: String): Option[Gauge[_]] = {
+    Option(registry.getGauges().get(name))
+  }
+
   def registerGauge[T](name: String, value: => T, default: T): Unit = {
     registry.register(
       MetricRegistry.name(name),
@@ -101,6 +105,8 @@ object MetricsSystem {
 
   @volatile private var maybeSystem: Option[MetricsSystem] = None
 
+  private[kyuubi] def getMetricsRegistry: Option[MetricRegistry] = maybeSystem.map(_.registry)
+
   def tracing[T](func: MetricsSystem => T): Unit = {
     maybeSystem.foreach(func(_))
   }
