@@ -35,9 +35,11 @@ import org.apache.kyuubi.config.KyuubiConf.{KubernetesApplicationStateSource, Ku
 import org.apache.kyuubi.config.KyuubiConf.KubernetesApplicationStateSource.KubernetesApplicationStateSource
 import org.apache.kyuubi.config.KyuubiConf.KubernetesCleanupDriverPodStrategy.{ALL, COMPLETED, NONE}
 import org.apache.kyuubi.engine.ApplicationState.{isTerminated, ApplicationState, FAILED, FINISHED, KILLED, NOT_FOUND, PENDING, RUNNING, UNKNOWN}
+import org.apache.kyuubi.engine.KubernetesApplicationUrlSource._
 import org.apache.kyuubi.engine.KubernetesResourceEventTypes.KubernetesResourceEventType
 import org.apache.kyuubi.operation.OperationState
 import org.apache.kyuubi.server.metadata.MetadataManager
+import org.apache.kyuubi.server.metadata.api.KubernetesEngineInfo
 import org.apache.kyuubi.util.{KubernetesUtils, ThreadUtils}
 
 class KubernetesApplicationOperation extends ApplicationOperation with Logging {
@@ -64,6 +66,13 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
   private def appStateContainer: String =
     kyuubiConf.get(KyuubiConf.KUBERNETES_APPLICATION_STATE_CONTAINER)
 
+  private lazy val sparkAppUrlPattern = kyuubiConf.get(KyuubiConf.KUBERNETES_SPARK_APP_URL_PATTERN)
+  private lazy val sparkAppUrlSource = if (sparkAppUrlPattern.contains(SPARK_DRIVER_SVC_PATTERN)) {
+    KubernetesApplicationUrlSource.SVC
+  } else {
+    KubernetesApplicationUrlSource.POD
+  }
+
   // key is kyuubi_unique_key
   private val appInfoStore: ConcurrentHashMap[String, (KubernetesInfo, ApplicationInfo)] =
     new ConcurrentHashMap[String, (KubernetesInfo, ApplicationInfo)]
@@ -107,7 +116,7 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
               if (appInfoStore.get(kyuubiUniqueKey) == null) {
                 updateApplicationState(kubernetesInfo, pod, eventType)
               }
-              markApplicationTerminated(pod, eventType)
+              markApplicationTerminated(kubernetesInfo, pod, eventType)
             }
           }
         }
@@ -142,11 +151,14 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
         val enginePodInformer = client.pods()
           .withLabel(LABEL_KYUUBI_UNIQUE_KEY)
           .inform(new SparkEnginePodEventHandler(kubernetesInfo))
-        info(s"[$kubernetesInfo] Start Kubernetes Client Informer.")
-        val engineSvcInformer = client.services()
-          .inform(new SparkEngineSvcEventHandler(kubernetesInfo))
+        info(s"[$kubernetesInfo] Start Kubernetes Client POD Informer.")
         enginePodInformers.put(kubernetesInfo, enginePodInformer)
-        engineSvcInformers.put(kubernetesInfo, engineSvcInformer)
+        if (sparkAppUrlSource == KubernetesApplicationUrlSource.SVC) {
+          info(s"[$kubernetesInfo] Start Kubernetes Client SVC Informer.")
+          val engineSvcInformer = client.services()
+            .inform(new SparkEngineSvcEventHandler(kubernetesInfo))
+          engineSvcInformers.put(kubernetesInfo, engineSvcInformer)
+        }
         client
 
       case None => throw new KyuubiException(s"Fail to build Kubernetes client for $kubernetesInfo")
@@ -172,7 +184,7 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
           val shouldDelete = cleanupDriverPodStrategy match {
             case NONE => false
             case ALL => true
-            case COMPLETED => !ApplicationState.isFailed(notification.getValue)
+            case COMPLETED => !ApplicationState.isFailed(notification.getValue, Some(this))
           }
           if (shouldDelete) {
             deletePod(kubernetesInfo, removed.podName.orNull, appLabel)
@@ -187,13 +199,8 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
       expireCleanUpTriggerCacheExecutor,
       () => {
         try {
-          Option(cleanupTerminatedAppInfoTrigger).foreach { trigger =>
-            trigger.asMap().asScala.foreach {
-              case (key, _) =>
-                // do get to trigger cache eviction
-                trigger.getIfPresent(key)
-            }
-          }
+          // Proactively remove expired entries from the cache
+          Option(cleanupTerminatedAppInfoTrigger).foreach(_.cleanUp())
         } catch {
           case NonFatal(e) => error("Failed to evict clean up terminated app cache", e)
         }
@@ -291,15 +298,27 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
       throw new IllegalStateException("Methods initialize and isSupported must be called ahead")
     }
     debug(s"Getting application[${toLabel(tag)}]'s info from Kubernetes cluster")
+    val startTime = System.currentTimeMillis()
     try {
       // need to initialize the kubernetes client if not exists
       getOrCreateKubernetesClient(appMgrInfo.kubernetesInfo)
-      val (_, appInfo) =
-        appInfoStore.getOrDefault(tag, appMgrInfo.kubernetesInfo -> ApplicationInfo.NOT_FOUND)
+      val appInfo = appInfoStore.get(tag) match {
+        case (_, info) => info
+        case _ =>
+          // try to get the application info from kubernetes engine info store
+          try {
+            metadataManager.flatMap(
+              _.getKubernetesApplicationInfo(tag)).getOrElse(ApplicationInfo.NOT_FOUND)
+          } catch {
+            case e: Exception =>
+              error(s"Failed to get application info from metadata manager for ${toLabel(tag)}", e)
+              ApplicationInfo.NOT_FOUND
+          }
+      }
       (appInfo.state, submitTime) match {
         // Kyuubi should wait second if pod is not be created
         case (NOT_FOUND, Some(_submitTime)) =>
-          val elapsedTime = System.currentTimeMillis - _submitTime
+          val elapsedTime = startTime - _submitTime
           if (elapsedTime > submitTimeout) {
             error(s"Can't find target driver pod by ${toLabel(tag)}, " +
               s"elapsed time: ${elapsedTime}ms exceeds ${submitTimeout}ms.")
@@ -322,6 +341,8 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
     }
   }
 
+  override def supportPersistedAppState: Boolean = true
+
   override def stop(): Unit = {
     enginePodInformers.asScala.foreach { case (_, informer) =>
       Utils.tryLogNonFatalError(informer.stop())
@@ -384,7 +405,7 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
         updateApplicationState(kubernetesInfo, newPod, eventType)
         val appState = toApplicationState(newPod, appStateSource, appStateContainer, eventType)
         if (isTerminated(appState)) {
-          markApplicationTerminated(newPod, eventType)
+          markApplicationTerminated(kubernetesInfo, newPod, eventType)
         }
         KubernetesApplicationAuditLogger.audit(
           eventType,
@@ -402,7 +423,7 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
       if (isSparkEnginePod(pod)) {
         val eventType = KubernetesResourceEventTypes.DELETE
         updateApplicationState(kubernetesInfo, pod, eventType)
-        markApplicationTerminated(pod, eventType)
+        markApplicationTerminated(kubernetesInfo, pod, eventType)
         KubernetesApplicationAuditLogger.audit(
           eventType,
           kubernetesInfo,
@@ -459,6 +480,9 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
             id = getPodAppId(pod),
             name = getPodAppName(pod),
             state = appState,
+            url = appInfo.url.orElse {
+              getPodAppUrl(sparkAppUrlSource, sparkAppUrlPattern, kubernetesInfo, pod)
+            },
             error = appError,
             podName = Some(pod.getMetadata.getName)))
       }.getOrElse {
@@ -468,6 +492,7 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
             id = getPodAppId(pod),
             name = getPodAppName(pod),
             state = appState,
+            url = getPodAppUrl(sparkAppUrlSource, sparkAppUrlPattern, kubernetesInfo, pod),
             error = appError,
             podName = Some(pod.getMetadata.getName)))
       }
@@ -485,7 +510,8 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
         val appUrl = buildSparkAppUrl(
           appUrlPattern,
           sparkAppId,
-          sparkDriverSvc,
+          sparkDriverSvc = Some(sparkDriverSvc),
+          sparkDriverPodIp = None,
           kubernetesContext,
           kubernetesNamespace,
           sparkUiPort)
@@ -500,13 +526,28 @@ class KubernetesApplicationOperation extends ApplicationOperation with Logging {
   }
 
   private def markApplicationTerminated(
+      kubernetesInfo: KubernetesInfo,
       pod: Pod,
       eventType: KubernetesResourceEventType): Unit = synchronized {
     val key = pod.getMetadata.getLabels.get(LABEL_KYUUBI_UNIQUE_KEY)
+    val (appState, appError) =
+      toApplicationStateAndError(pod, appStateSource, appStateContainer, eventType)
+    // upsert the kubernetes engine info store when the application is terminated
+    metadataManager.foreach(_.upsertKubernetesMetadata(
+      KubernetesEngineInfo(
+        identifier = key,
+        context = kubernetesInfo.context,
+        namespace = kubernetesInfo.namespace,
+        podName = pod.getMetadata.getName,
+        podState = pod.getStatus.getPhase,
+        containerState = pod.getStatus.getContainerStatuses.asScala.map(cs =>
+          s"${cs.getName}->${cs.getState}").mkString(","),
+        engineId = getPodAppId(pod),
+        engineName = getPodAppName(pod),
+        engineState = appState.toString,
+        engineError = appError)))
     if (cleanupTerminatedAppInfoTrigger.getIfPresent(key) == null) {
-      cleanupTerminatedAppInfoTrigger.put(
-        key,
-        toApplicationState(pod, appStateSource, appStateContainer, eventType))
+      cleanupTerminatedAppInfoTrigger.put(key, appState)
     }
   }
 
@@ -618,8 +659,8 @@ object KubernetesApplicationOperation extends Logging {
       case Some(containerAppState) => containerAppState
       case None => podAppState
     }
-    val applicationError =
-      if (ApplicationState.isFailed(applicationState)) {
+    val applicationError = {
+      if (ApplicationState.isFailed(applicationState, supportPersistedAppState = true)) {
         val errorMap = containerStatusToBuildAppState.map { cs =>
           Map(
             "Pod" -> podName,
@@ -633,6 +674,7 @@ object KubernetesApplicationOperation extends Logging {
       } else {
         None
       }
+    }
     applicationState -> applicationError
   }
 
@@ -666,9 +708,17 @@ object KubernetesApplicationOperation extends Logging {
       UNKNOWN
   }
 
+  final private val SPARK_APP_ID_PATTERN = "{{SPARK_APP_ID}}"
+  final private val SPARK_DRIVER_SVC_PATTERN = "{{SPARK_DRIVER_SVC}}"
+  final private val SPARK_DRIVER_POD_IP_PATTERN = "{{SPARK_DRIVER_POD_IP}}"
+  final private val KUBERNETES_CONTEXT_PATTERN = "{{KUBERNETES_CONTEXT}}"
+  final private val KUBERNETES_NAMESPACE_PATTERN = "{{KUBERNETES_NAMESPACE}}"
+  final private val SPARK_UI_PORT_PATTERN = "{{SPARK_UI_PORT}}"
+
   /**
    * Replaces all the {{SPARK_APP_ID}} occurrences with the Spark App Id,
    * {{SPARK_DRIVER_SVC}} occurrences with the Spark Driver Service name,
+   * {{SPARK_DRIVER_POD_IP}} occurrences with the Spark Driver Pod IP,
    * {{KUBERNETES_CONTEXT}} occurrences with the Kubernetes Context,
    * {{KUBERNETES_NAMESPACE}} occurrences with the Kubernetes Namespace,
    * and {{SPARK_UI_PORT}} occurrences with the Spark UI Port.
@@ -676,22 +726,61 @@ object KubernetesApplicationOperation extends Logging {
   private[kyuubi] def buildSparkAppUrl(
       sparkAppUrlPattern: String,
       sparkAppId: String,
-      sparkDriverSvc: String,
+      sparkDriverSvc: Option[String],
+      sparkDriverPodIp: Option[String],
       kubernetesContext: String,
       kubernetesNamespace: String,
       sparkUiPort: Int): String = {
-    sparkAppUrlPattern
-      .replace("{{SPARK_APP_ID}}", sparkAppId)
-      .replace("{{SPARK_DRIVER_SVC}}", sparkDriverSvc)
-      .replace("{{KUBERNETES_CONTEXT}}", kubernetesContext)
-      .replace("{{KUBERNETES_NAMESPACE}}", kubernetesNamespace)
-      .replace("{{SPARK_UI_PORT}}", sparkUiPort.toString)
+    var appUrl = sparkAppUrlPattern
+      .replace(SPARK_APP_ID_PATTERN, sparkAppId)
+      .replace(KUBERNETES_CONTEXT_PATTERN, kubernetesContext)
+      .replace(KUBERNETES_NAMESPACE_PATTERN, kubernetesNamespace)
+      .replace(SPARK_UI_PORT_PATTERN, sparkUiPort.toString)
+    sparkDriverSvc match {
+      case Some(svc) => appUrl = appUrl.replace(SPARK_DRIVER_SVC_PATTERN, svc)
+      case None =>
+    }
+    sparkDriverPodIp match {
+      case Some(ip) => appUrl = appUrl.replace(SPARK_DRIVER_POD_IP_PATTERN, ip)
+      case None =>
+    }
+    appUrl
   }
 
   def getPodAppId(pod: Pod): String = {
     pod.getMetadata.getLabels.get(SPARK_APP_ID_LABEL)
   }
 
+  private[kyuubi] def getPodAppUrl(
+      sparkAppUrlSource: KubernetesApplicationUrlSource,
+      sparkAppUrlPattern: String,
+      kubernetesInfo: KubernetesInfo,
+      pod: Pod): Option[String] = {
+    sparkAppUrlSource match {
+      case KubernetesApplicationUrlSource.SVC => None
+      case KubernetesApplicationUrlSource.POD =>
+        val podIp = Option(pod.getStatus.getPodIP).filter(_.nonEmpty)
+        podIp match {
+          case Some(ip) => pod.getSpec.getContainers.asScala.flatMap(
+              _.getPorts.asScala).find(_.getName == SPARK_UI_PORT_NAME)
+              .map(_.getContainerPort).map { sparkUiPort =>
+                buildSparkAppUrl(
+                  sparkAppUrlPattern,
+                  getPodAppId(pod),
+                  sparkDriverSvc = None,
+                  sparkDriverPodIp = Some(ip),
+                  kubernetesContext = kubernetesInfo.context.orNull,
+                  kubernetesNamespace = kubernetesInfo.namespace.orNull,
+                  sparkUiPort = sparkUiPort)
+              }.orElse {
+                warn(s"Spark UI port not found in pod ${pod.getMetadata.getName}")
+                None
+              }
+          case None => None
+        }
+    }
+  }
+
   def getPodAppName(pod: Pod): String = {
     Option(pod.getMetadata.getLabels.get(SPARK_APP_NAME_LABEL)).getOrElse(pod.getMetadata.getName)
   }
