@@ -68,6 +68,7 @@ private[v1] class BatchesResource extends ApiRequestContext with Logging {
   private lazy val resourceFileMaxSize = fe.getConf.get(BATCH_RESOURCE_FILE_MAX_SIZE)
   private lazy val extraResourceFileMaxSize = fe.getConf.get(BATCH_EXTRA_RESOURCE_FILE_MAX_SIZE)
   private lazy val metadataSearchWindow = fe.getConf.get(METADATA_SEARCH_WINDOW)
+  private lazy val batchInfoInternalRedirect = fe.getConf.get(BATCH_INFO_INTERNAL_REDIRECT)
 
   private def batchV2Enabled(reqConf: Map[String, String]): Boolean = {
     fe.getConf.get(BATCH_SUBMITTER_ENABLED) &&
@@ -128,8 +129,9 @@ private[v1] class BatchesResource extends ApiRequestContext with Logging {
       metadata: Metadata,
       batchAppStatus: Option[ApplicationInfo]): Batch = {
     batchAppStatus.map { appStatus =>
+      val appOp = sessionManager.applicationManager.getApplicationOperation(metadata.appMgrInfo)
       val currentBatchState =
-        if (BatchJobSubmission.applicationFailed(batchAppStatus)) {
+        if (BatchJobSubmission.applicationFailed(batchAppStatus, appOp)) {
           OperationState.ERROR.toString
         } else if (BatchJobSubmission.applicationTerminated(batchAppStatus)) {
           OperationState.FINISHED.toString
@@ -355,7 +357,8 @@ private[v1] class BatchesResource extends ApiRequestContext with Logging {
         val isApplicationTerminated = (StringUtils.isNotBlank(metadata.engineState)
           && ApplicationState.isTerminated(ApplicationState.withName(metadata.engineState)))
 
-        if (batchV2Enabled(metadata.requestConf) ||
+        if (!batchInfoInternalRedirect ||
+          batchV2Enabled(metadata.requestConf) ||
           isOperationTerminated ||
           isApplicationTerminated ||
           metadata.kyuubiInstance == fe.connectionUrl) {
@@ -572,6 +575,42 @@ private[v1] class BatchesResource extends ApiRequestContext with Logging {
     }
   }
 
+  @ApiResponse(
+    responseCode = "200",
+    content = Array(new Content(
+      mediaType = MediaType.APPLICATION_JSON,
+      schema = new Schema(implementation = classOf[ReassignBatchResponse]))),
+    description =
+      "Reassign batch sessions on an unreachable kyuubi instance to the current kyuubi instance")
+  @POST
+  @Path("/reassign")
+  @Consumes(Array(MediaType.APPLICATION_JSON))
+  def reassignBatchSessions(request: ReassignBatchRequest): ReassignBatchResponse = {
+    val userName = fe.getSessionUser(Map.empty[String, String])
+    val ipAddress = fe.getIpAddress
+    val kyuubiInstance = request.getKyuubiInstance
+    val newKyuubiInstance = fe.connectionUrl
+    info(s"Received reassign $kyuubiInstance batch sessions request from $userName/$ipAddress")
+    if (!fe.isAdministrator(userName)) {
+      throw new ForbiddenException(s"$userName is not allowed to reassign the batches")
+    }
+    if (kyuubiInstance == newKyuubiInstance) {
+      throw new IllegalStateException(s"KyuubiInstance is alive: $kyuubiInstance")
+    }
+    val internalRestClient = getInternalRestClient(kyuubiInstance)
+    if (!Utils.isTesting && internalRestClient.pingAble(userName, ipAddress)) {
+      throw new IllegalStateException(s"KyuubiInstance is alive: $kyuubiInstance")
+    }
+    try {
+      val batchIds = sessionManager.reassignBatchSessions(kyuubiInstance, newKyuubiInstance)
+      val recoveredBatchIds = fe.recoverBatchSessionsFromReassign(batchIds)
+      new ReassignBatchResponse(recoveredBatchIds.asJava, kyuubiInstance, newKyuubiInstance)
+    } catch {
+      case e: Exception =>
+        throw new Exception(s"Error reassign batches from $kyuubiInstance to $newKyuubiInstance", e)
+    }
+  }
+
   private def handleUploadingFiles(
       batchId: String,
       request: BatchRequest,
