@@ -20,8 +20,10 @@ package org.apache.kyuubi.server.api.v1
 import java.util.Base64
 
 import scala.collection.JavaConverters._
+import scala.util.{Failure, Success, Try}
 
-import org.apache.kyuubi.client.{BatchRestApi, KyuubiRestClient}
+import org.apache.kyuubi.Logging
+import org.apache.kyuubi.client.{BaseRestApi, BatchRestApi, KyuubiRestClient}
 import org.apache.kyuubi.client.api.v1.dto.{Batch, CloseBatchResponse, OperationLog}
 import org.apache.kyuubi.client.auth.AuthHeaderGenerator
 import org.apache.kyuubi.server.http.authentication.AuthSchemes
@@ -45,7 +47,7 @@ class InternalRestClient(
     connectTimeout: Int,
     securityEnabled: Boolean,
     requestMaxAttempts: Int,
-    requestAttemptWait: Int) {
+    requestAttemptWait: Int) extends Logging {
   if (securityEnabled) {
     require(
       InternalSecurityAccessor.get() != null,
@@ -53,6 +55,18 @@ class InternalRestClient(
   }
 
   private val internalBatchRestApi = new BatchRestApi(initKyuubiRestClient())
+  private val internalBaseRestApi = new BaseRestApi(initKyuubiRestClient())
+
+  def pingAble(user: String, clientIp: String): Boolean = withAuthUser(user) {
+    Try {
+      internalBaseRestApi.ping(Map(proxyClientIpHeader -> clientIp).asJava)
+    } match {
+      case Success(_) => true
+      case Failure(e) =>
+        error(s"Ping to Kyuubi instance $kyuubiInstance failed", e)
+        false
+    }
+  }
 
   def getBatch(user: String, clientIp: String, batchId: String): Batch = {
     withAuthUser(user) {
