@@ -259,7 +259,7 @@ class BatchCliSuite extends RestClientTestHelper with TestPrematureExit with Bat
       "--forward")
     result = testPrematureExitForControlCli(logArgs, "")
     assert(result.contains(s"Submitted application: $sparkBatchTestAppName"))
-    assert(result.contains("Shutdown hook called"))
+    assert(result.contains("Successfully stopped SparkContext"))
   }
 
   test("submit batch test") {
@@ -272,7 +272,7 @@ class BatchCliSuite extends RestClientTestHelper with TestPrematureExit with Bat
       ldapUserPasswd)
     val result = testPrematureExitForControlCli(submitArgs, "")
     assert(result.contains(s"Submitted application: $sparkBatchTestAppName"))
-    assert(result.contains("Shutdown hook called"))
+    assert(result.contains("Successfully stopped SparkContext"))
   }
 
   test("submit batch test with waitCompletion=false") {
@@ -289,7 +289,7 @@ class BatchCliSuite extends RestClientTestHelper with TestPrematureExit with Bat
       s"${CtlConf.CTL_BATCH_LOG_QUERY_INTERVAL.key}=100")
     val result = testPrematureExitForControlCli(submitArgs, "")
     assert(result.contains("bin/spark-submit"))
-    assert(!result.contains("Shutdown hook called"))
+    assert(!result.contains("Successfully stopped SparkContext"))
   }
 
   test("list batch test") {
