@@ -47,5 +47,10 @@ export const MENUS = [
     label: 'Swagger',
     icon: 'List',
     router: '/swagger'
+  },
+  {
+    label: 'SQL Editor',
+    icon: 'Cpu',
+    router: '/editor'
   }
 ]
